@@ -1,6 +1,6 @@
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 // Mobius Forensic Toolkit
-// Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018 Eduardo Aguiar
+// Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018,2019 Eduardo Aguiar
 //
 // This program is free software; you can redistribute it and/or modify it
 // under the terms of the GNU General Public License as published by the
@@ -21,6 +21,7 @@
 #include <sqlite3.h>
 #include <thread>
 #include <chrono>
+#include <cstring>
 
 namespace mobius
 {
@@ -29,37 +30,37 @@ namespace database
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 //! \brief implementation struct
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-struct statement_impl
+struct statement::impl
 {
   database db;
   sqlite3_stmt *stmt = nullptr;
-  
-  ~statement_impl ();
+
+  ~impl ();
 };
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 //! \brief destroy shared implementation
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-statement_impl::~statement_impl ()
+statement::impl::~impl ()
 {
-  if (stmt)
+  if (stmt != nullptr)
     sqlite3_finalize (stmt);
 }
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 //! \brief create statement object
-// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=        
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 statement::statement ()
 {
-  impl_ = std::make_shared <statement_impl> ();
+  impl_ = std::make_shared <impl> ();
 }
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 //! \brief create statement object
-// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=        
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 statement::statement (database db, sqlite3_stmt *stmt)
 {
-  impl_ = std::make_shared <statement_impl> ();
+  impl_ = std::make_shared <impl> ();
   impl_->stmt = stmt;
   impl_->db = db;
 }
@@ -68,7 +69,7 @@ statement::statement (database db, sqlite3_stmt *stmt)
 //! \brief bind int value
 //! \param idx value index
 //! \param value value
-// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=        
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 void
 statement::bind (int idx, int value)
 {
@@ -76,6 +77,18 @@ statement::bind (int idx, int value)
     throw std::runtime_error (MOBIUS_EXCEPTION_SQLITE);
 }
 
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief bind uint64_t value
+//! \param idx value index
+//! \param value value
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+void
+statement::bind (int idx, std::int64_t value)
+{
+  if (sqlite3_bind_int64 (impl_->stmt, idx, value) != SQLITE_OK)
+    throw std::runtime_error (MOBIUS_EXCEPTION_SQLITE);
+}
+
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 //! \brief bind double value
 //! \param idx value index
@@ -88,6 +101,18 @@ statement::bind (int idx, double value)
     throw std::runtime_error (MOBIUS_EXCEPTION_SQLITE);
 }
 
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief bind C string value
+//! \param idx value index
+//! \param v value
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+void
+statement::bind (int idx, const char *value)
+{
+  if (sqlite3_bind_text (impl_->stmt, idx, value, strlen (value), SQLITE_STATIC) != SQLITE_OK)
+    throw std::runtime_error (MOBIUS_EXCEPTION_SQLITE);
+}
+
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 //! \brief bind string value
 //! \param idx value index
@@ -100,6 +125,18 @@ statement::bind (int idx, const std::string& value)
     throw std::runtime_error (MOBIUS_EXCEPTION_SQLITE);
 }
 
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief bind bytearray value
+//! \param idx value index
+//! \param v value
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+void
+statement::bind (int idx, const mobius::bytearray& value)
+{
+  if (sqlite3_bind_blob (impl_->stmt, idx, value.begin (), value.size (), SQLITE_STATIC) != SQLITE_OK)
+    throw std::runtime_error (MOBIUS_EXCEPTION_SQLITE);
+}
+
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 //! \brief bind null value
 //! \param idx value index
@@ -112,16 +149,16 @@ statement::bind_null (int idx)
 }
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-//! \brief exec statement
+//! \brief execute statement
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 void
-statement::exec ()
+statement::execute ()
 {
   int rc = step ();
- 
+
   if (rc == SQLITE_DONE)
     sqlite3_reset (impl_->stmt);
-    
+
   else
     throw std::runtime_error (MOBIUS_EXCEPTION_SQLITE);
 }
@@ -138,13 +175,13 @@ statement::fetch_row ()
 
   // check if a row was fetched
   bool has_row = false;
-  
+
   if (rc == SQLITE_DONE)
     sqlite3_reset (impl_->stmt);
 
   else if (rc == SQLITE_ROW)
     has_row = true;
-    
+
   else
     throw std::runtime_error (MOBIUS_EXCEPTION_SQLITE);
 
@@ -181,6 +218,16 @@ statement::get_column_int (int idx)
   return sqlite3_column_int (impl_->stmt, idx);
 }
 
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief get int64_t column value
+//! \param idx value index
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+std::int64_t
+statement::get_column_int64 (int idx)
+{
+  return sqlite3_column_int64 (impl_->stmt, idx);
+}
+
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 //! \brief get double column value
 //! \param idx value index
@@ -195,15 +242,33 @@ statement::get_column_double (int idx)
 //! \brief get string column value
 //! \param idx value index
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-const std::string
+std::string
 statement::get_column_string (int idx)
 {
-  const char *p_text = (const char *) sqlite3_column_text (impl_->stmt, idx);
+  const char *p_text = reinterpret_cast<const char *> (sqlite3_column_text (impl_->stmt, idx));
   std::string value;
 
-  if (p_text)
+  if (p_text != nullptr)
     value = std::string (p_text);
-  
+
+  return value;
+}
+
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief get bytearray column value
+//! \param idx value index
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+mobius::bytearray
+statement::get_column_bytearray (int idx)
+{
+  mobius::bytearray value;
+
+  auto size = sqlite3_column_bytes (impl_->stmt, idx);
+  const void *p_blob = sqlite3_column_blob (impl_->stmt, idx);
+
+  if (p_blob && size > 0)
+    value = mobius::bytearray (reinterpret_cast <const uint8_t *> (p_blob), size);
+
   return value;
 }
 
@@ -211,7 +276,7 @@ statement::get_column_string (int idx)
 //! \brief get database error message
 //! \return error message
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-const std::string
+std::string
 statement::get_error_message () const
 {
   return impl_->db.get_error_message ();
@@ -224,17 +289,17 @@ statement::get_error_message () const
 int
 statement::step ()
 {
-  int rc = SQLITE_BUSY;
   constexpr int SLEEP_TIME = 100;        // microseconds
-  
+  int rc = SQLITE_BUSY;
+
   while (rc == SQLITE_BUSY)
     {
       rc = sqlite3_step (impl_->stmt);
-  
+
       if (rc == SQLITE_BUSY)
         std::this_thread::sleep_for (std::chrono::microseconds (SLEEP_TIME));
     }
-    
+
   return rc;
 }
 
